﻿using System;
using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Controls;
using ThinkGeo.MapSuite.SilverlightCore;
using ThinkGeo.MapSuite.SilverlightEdition;

namespace DraggableMarker
{
    public class DraggableSimpleMarkerOverlay : SimpleMarkerOverlay
    {
        private Marker selectedMarker;
        private double dragOffsetX;
        private double dragOffsetY;

        public DraggableSimpleMarkerOverlay()
            : this(Guid.NewGuid().ToString())
        { }
        
        public DraggableSimpleMarkerOverlay(string id)
            : base(id)
        { }

        protected override Collection<Marker> GetDrawingMarkersCore(RectangleShape worldExtent)
        {
            Collection<Marker> returnMarkers = new Collection<Marker>();
            if (Markers != null)
            {
                foreach (Marker marker in Markers)
                {
                    if (worldExtent.Contains(new PointShape(marker.Position.X, marker.Position.Y)))
                    {
                        marker.MouseLeftButtonDown += new System.Windows.Input.MouseButtonEventHandler(marker_MouseLeftButtonDown);
                        returnMarkers.Add(marker);
                    }
                }
            }
            return returnMarkers;
        }

        protected override void InitializeCore()
        {
            base.InitializeCore();
            MapInfo.MouseMove += new System.Windows.Input.MouseEventHandler(markerCanvas_MouseMove);
            MapInfo.MouseLeftButtonUp += new System.Windows.Input.MouseButtonEventHandler(markerCanvas_MouseLeftButtonUp);
        }

        void markerCanvas_MouseLeftButtonUp(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            if (selectedMarker != null)
            {
                double screenX = (double)selectedMarker.GetValue(Canvas.LeftProperty);
                double screenY = (double)selectedMarker.GetValue(Canvas.TopProperty);

                PointShape markerWorldPosition = MapInfo.ToWorldCoordinate(new Point(screenX, screenY));
                selectedMarker.Position = new Point(markerWorldPosition.X, markerWorldPosition.Y);

                selectedMarker = null;
                e.Handled = true;
            }
        }

        void markerCanvas_MouseMove(object sender, System.Windows.Input.MouseEventArgs e)
        {
            if (selectedMarker != null)
            {
                Canvas canvas = (Canvas)sender;
                Point mousePosition = e.GetPosition(canvas);
                selectedMarker.SetValue(Canvas.LeftProperty, mousePosition.X - dragOffsetX);
                selectedMarker.SetValue(Canvas.TopProperty, mousePosition.Y - dragOffsetY);
            }
        }

        void marker_MouseLeftButtonDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            selectedMarker = (Marker)sender;
            Point dragPoint = e.GetPosition(selectedMarker);
            dragOffsetX = dragPoint.X;
            dragOffsetY = dragPoint.Y;
            e.Handled = true;
        }
    }
}
